#!/bin/bash
# NAME:  FAA_GEN_alignment_pipeline_gencode_v1.2_PM.sh
# USAGE: sh ~/supplemental_file_1_FAA_GEN_alignment_pipeline_gencode_v1.2_PM.sh
# Christopher J. Tracy
# July 14, 2023
#
## The purpose of this script is to take a directory of split paired-end fastq.gz files from an RNA-seq experiment, perform quality assessment on them including trimming and quality filtering, perform taxonomic analysis (PlusPF), then align them by coordinate and to transcriptome against a previously indexed reference genome (Gencode), perform a final round of quality assessment on the alignments, and then output a featurecounts matrix for further processing in Vim.
#
# Step 0.  Defines reference directories and variables.
# Step 1.  Run FastQC on .fastq.gz files within a directory; run MultiQC on the FastQC output directory.
# Step 2.  Run CutAdapt on the .fastq.gz files from Step 1.
# Step 3.  Run FastQC on the .fastq.gz files from Step 2; run MultiQC on the FastQC output directory.
# Step 4.  Run STAR on the .fastq.gz files from Step 2; run Samtools index on the Aligned.sortedByCoord.out.bam files from STAR.
# Step 5.  Run Kraken2 on the unmapped .mate files from Step 4; run Bracken, Krona, and MultiQC on the output of Kraken2.
# Step 6.  Run Qualimap on the Aligned.sortedByCoord.bam files from Step 6.  Preseq, RNA-SeQC, RSeQC, Picard, DeepTools, Rsem, and Salmon are currently not in use (2023.07.06).
# Step 7.  Run Subread featureCounts on the Aligned.sortedByCoord.out.bam files from Step 5.
# Step 8.  Run MultiQC on the output directories of Steps 4, 5, 6, and 7.
# Step 9.  Run MultiQC on the output directories of Steps 1, 2, 3, 4, 5, 6, and 7.
#
## This script makes use of the following software:
# FastQC-0.12.1
# CutAdapt-4.3; installed through Conda, uses the "cutadapt" environment
# Kraken v1.1.1, Kraken2 v2.1.2, and Bracken v2.8; installed through Conda, uses the "kraken" environment
# KronaTools-2.8.1
#	requires running scripts in the KronaTools installation folder to update Taxonomy and Accessions prior to first use
# STAR-2.7.10.b
# Samtools-1.17
# QualiMap v.2.2.2d; installed through Conda, uses the "qualimap" environment
# deepTools-3.5.1; installed through Conda, uses the "deeptools" environment
# Preseq v3.2.0; installed through Conda, uses the "preseq" environment
# RNA-SeQC v2.3.5; installed through Conda, uses the "rna-seqc" environment
#	requires a collapsed GTF file generated using the "bx-python" environment
# RSeQC v5.0.1; installed through Conda, uses the "rseqc" environment
#	requires a BED file generated using either the "bedops" or "ucsc" environment (use "ucsc" for Ensembl/Gencode GTFs)
# Picard v2.27.5; installed throuhg Conda, uses the "picard" environment
#	requires a refFlat file generated using the "ucsc" environment and then physically edited in LibreOffice Calc
#	requires an rRNA intervals file physically edited in Text Editor
# Salmon v1.10.1; installed through Conda, uses the "salmon" environment
# Rsem v1.3.3; installed through Conda, uses the "rsem" environment
# Subread-2.0.4
# MultiQC-1.14; installed through Conda, uses the "multiqc" environment
# Other packages that may be implemented in the future:
# 	FastQ-Screen v0.15.2; Conda (this tool is severely out of date and will likely not be implemented in this script)
# 
## Note:  All file paths are specific to a user's computer and will need to be tailored to each machine.
#
## Note:  This pipeline uses the human Gencode implementation of the Ensembl reference genome, GRCh38.p13.43, indexed with a 149 bp sjdb overhang.
## Note:  This pipeline uses the PlusPF index collection (archaea, bacteria, viral, plasmid, human, UniVec_Core, plasmid, and fungi) for taxonomic analysis.

## Captures the terminal output within the "terminal_log.txt" file.
exec > /<file_path>/paired/terminal_log.txt 2>&1
set -x
## Note:  In order to visualize the progress of this file being written to, in another terminal window, run the following command:
## $ tail -f /<file_path>/paired/terminal_log.txt
## Note:  Once the script is finished running in the other terminal window, use the keyboard shortcut ctrl+C to stop the tail command.

echo "This alignment pipeline is using the script FAA_GEN_alignment_pipeline_gencode_v1.2_PM.sh."
date

## Step 0 Setup ----------------------------------------------------------------------------------------------------------------
# Defines the taxon database directory for use with Kraken2 and Bracken.
taxon_pluspf=/<file_path>/Data/reference_data/kraken2
# Defines the genome directory for use with STAR.
genome_gencode=/<file_path>/Data/reference_data/gencode_GRCh38.p13.43_150bp
# Defines the gtf variable for use with Qualimap and featureCounts.
gtf_gencode=/<file_path>/Data/reference_data/gencode_GRCh38.p13.43_150bp/gencode.v43.primary_assembly.annotation.gtf
## Defines the collapsed gtf variable for use with RNA-SeQC.
#gtf_gencode_collapsed=/<file_path>/Data/reference_data/gencode_GRCh38.p13.43_150bp/gencode.v43.primary_assembly.annotation.collapsed.gtf
## Defines the bed converted gtf variable for use with RSeQC.
#gtf_gencode_bed=/<file_path>/Data/reference_data/gencode_GRCh38.p13.43_150bp/gencode.v43.primary_assembly.annotation.bed
## Defines the refFlat converted gtf variable for use with Picard.
#gtf_gencode_refFlat=/<file_path>/Data/reference_data/gencode_GRCh38.p13.43_150bp/gencode.v43.primary_assembly.annotation.refFlat
## Defines the transcriptome variable for use with Salmon.
#transcripts_gencode=/<file_path>/Data/reference_data/gencode_GRCh38.p13.43_150bp/gencode.v43.transcriptome.fa
## Defines the transcript reference directory for use with Rsem.
#ref_gencode=/<file_path>/Data/reference_data/gencode_GRCh38.p13.43_150bp/ref/gencode.v43

## Step 1 Pre-Trimming QC ------------------------------------------------------------------------------------------------------
# Runs FastQC on the raw split paired-end fastq.gz files and outputs a FastQC report for each file, then runs MultiQC to collate the reports.

## Step 1.1 FastQC -------------------------------------------------------------------------------------------------------------
# A FastQC manual can be found at the following webpage:  https://dnacore.missouri.edu/PDF/FastQC_Manual.pdf
# Creates the directory for the FastQC reports and then changes directories to /<file_path>/paired/raw_files.
mkdir -p /<file_path>/paired/results/fastqc/before_trimming
cd /<file_path>/paired/raw_files
# Sets the output folder for the FastQC command.
fastqc_out=/<file_path>/paired/results/fastqc/before_trimming
# Runs the FastQC command using 72 threads in parallel on all the fastq.gz files in the raw_files directory.
find . -name "*.fastq.gz" | xargs fastqc -t 72 -o $fastqc_out

## Step 1.2 MultiQC ------------------------------------------------------------------------------------------------------------
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<file_path>/paired/results/fastqc/before_trimming.
cd /<file_path>/paired/results/fastqc/before_trimming
# Runs multiqc in the current directory.
multiqc -n before_trimming_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

## Step 2 Adapter Removal and Quality Filtering and Trimming -------------------------------------------------------------------
# Runs CutAdapt to trim and quality filter the raw paired-end fastq.gz files and outputs a report for each file in addition to the trimmed and filtered fastq files, then runs MultiQC to collate the reports.

## Step 2.1 Cutadapt -----------------------------------------------------------------------------------------------------------
# The CutAdapt manual can be found at the following webpage:  https://cutadapt.readthedocs.io/en/stable/guide.html
## Note:  This process effectively duplicates the contents of the raw_files directory, so it is imperative to ensure that there is sufficient space on the hard drive for all of the raw and output files (at least double the size of the raw_files directory).
# Activates the "cutadapt" Conda environment which CutAdapt runs within.
eval "$(conda shell.bash hook)"
conda activate cutadapt
# Creates the directory for the output of CutAdapt.  
mkdir -p /<file_path>/paired/results/cutadapt/
cd /<file_path>/paired/raw_files/
# Establishes the naming parameters for the for loop.
f1=$(ls *_1.fastq.gz | sed 's/_1.fastq.gz//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the fastq.gz files within the raw_files directory.
for fn in $f1
do
samplename=`basename ${fn}`
echo "Sample name is $samplename"
cutadapt_out=/<file_path>/paired/results/cutadapt
echo "Starting trimming and filtering for $samplename using CutAdapt."
# Runs the CutAdapt command in multi-core support mode with 72 cores.  Removes any reads shorter than 50 bp and trims any bases with a PHRED score of lower than 20.  This will result in some of the individual reads no longer being all the same length (for example, prior to trimming, reads could all be 150 bp; after trimming, depending on how many bases are trimmed, reads could range from 147-150 bp).
cutadapt -j 72 \
-a AGATCGGAAGAGCACACGTCTGAACTCCAGTCA \
-A AGATCGGAAGAGCGTCGTGTAGGGAAAGAGTGT \
--nextseq-trim=20 \
-m 50 \
-o $cutadapt_out/${fn}_trimmed_1.fastq.gz \
-p $cutadapt_out/${fn}_trimmed_2.fastq.gz \
${fn}_1.fastq.gz ${fn}_2.fastq.gz \
1> /<file_path>/paired/results/cutadapt/${fn}_report.txt
done
# Deactivates the "cutadapt" Conda environment.
conda deactivate

## Step 2.2 MultiQC ------------------------------------------------------------------------------------------------------------
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<file_path>/paired/results/cutadapt.
cd /<file_path>/paired/results/cutadapt
# Runs multiqc in the current directory.
multiqc -n cutadapt_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

## Step 3 Post-Trimming QC -----------------------------------------------------------------------------------------------------
# Runs FastQC on the trimmed and quality-filtered fastq.gz files from Step 2 and outputs a FastQC report for each file, then runs MultiQC to collate the reports.

## Step 3.1 FastQC -------------------------------------------------------------------------------------------------------------
# Creates the directory for the FastQC reports and then changes directories to /<file_path>/paired/results/cutadapt.
mkdir -p /<file_path>/paired/results/fastqc/after_trimming
cd /<file_path>/paired/results/cutadapt
# Sets the output folder for the FastQC command.
fastqc_out=/<file_path>/paired/results/fastqc/after_trimming
# Runs the FastQC command using 72 threads in parallel on all the fastq files in the cutadapt directory.
find . -name "*.fastq.gz" | xargs fastqc -t 72 -o $fastqc_out

## Step 3.2 MultiQC ------------------------------------------------------------------------------------------------------------
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<file_path>/paired/results/fastqc/after_trimming.
cd /<file_path>/paired/results/fastqc/after_trimming
# Runs multiqc in the current directory.
multiqc -n after_trimming_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

## Step 4 Alignment and Indexing -----------------------------------------------------------------------------------------------
# Aligns the trimmed and quality-filtered fastq.gz files from Step 2 using STAR with a previously STAR-indexed reference genome, then outputs the alignments into Aligned.sortedByCoord.out.bam and Aligned.toTranscriptome.out.bam files for each fastq.gz.  Then uses Samtools to create a .bai index for each Aligned.sortedByCoord.out.bam file.

## Step 4.1 STAR Alignment -----------------------------------------------------------------------------------------------------
# Modifies the number of temporary files allowed to be open at a single time so that STAR will run properly.
ulimit -Sn 24576
# Creates the directories for the output of STAR.
mkdir -p /<file_path>/paired/results/gencode/STAR/
# Changes directories to /<file_path>/paired/results/cutadapt and establishes the naming convention for the for loop.
cd /<file_path>/paired/results/cutadapt/
f1=$(ls *_trimmed_1.fastq.gz | sed 's/_trimmed_1.fastq.gz//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the fastq.gz files within the cutadapt directory.
for fn in $f1
do
samplename=`basename ${fn}`
echo "Sample name is $samplename."
# Defines the sample output directory variables for STAR.
align_out_gencode=/<file_path>/paired/results/gencode/STAR/${samplename}_
# Runs STAR in paired-end mode.
# The STAR manual can be found at the following webpage:  https://github.com/alexdobin/STAR/blob/master/doc/STARmanual.pdf
echo "Starting STAR run for paired-end sample set $samplename using gencode reference."
STAR --runThreadN 72 \
--genomeDir $genome_gencode \
--readFilesIn ${fn}_trimmed_1.fastq.gz ${fn}_trimmed_2.fastq.gz \
--readFilesCommand zcat \
--outFileNamePrefix $align_out_gencode \
--outSAMtype BAM SortedByCoordinate \
--outMultimapperOrder Random \
--outSAMattributes Standard \
--outReadsUnmapped Fastx
#--quantMode TranscriptomeSAM
done

## Step 4.2 BAM Indexing with Samtools -----------------------------------------------------------------------------------------
# Changes directories to /<file_path>/paired/results/gencode/STAR.
cd /<file_path>/paired/results/gencode/STAR/
echo "Starting indexing of the gencode BAM files using Samtools."
# Establishes the naming conventions for the for loop for Samtools.
f1=$(ls *_Aligned.sortedByCoord.out.bam | sed 's/_Aligned.sortedByCoord.out.bam//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the Aligned.sortedByCoord.out.bam files within the STAR directory.
for fn in $f1
do
samplename=`basename ${fn}`
echo "Sample name is $samplename."
echo "Indexing the BAM file for $samplename."
# Runs Samtools index with 72 threads.
# The documentation for Samtools index can be found at the following webpage:  http://www.htslib.org/doc/samtools-index.html
samtools index ${fn}_Aligned.sortedByCoord.out.bam -@ 72
done

## Step 5 Taxonomic Investigation ----------------------------------------------------------------------------------------------
# Runs fastq.gz files with Kraken2 and the PlusPF collection (archaea, bacteria, viral, plasmid, human, UniVec_Core, plasmid, and fungi).  The Kraken2 output is then run with Bracken.  The results are then visualized with Krona.  The output of all three tools is then collated with MultiQC.
# Overview on how to run taxonomic analysis at the following webpage:  https://genomics.sschmeier.com/ngs-taxonomic-investigation/index.html
# Indecies available at the following webpage:  https://benlangmead.github.io/aws-indexes/k2
# Krona installation instructions at the following webpage:  https://github.com/marbl/Krona/wiki/Installing

## Step 5.1 Kraken2 ------------------------------------------------------------------------------------------------------------
# Creates the directory for the taxonomic analysis results and Kraken2 reports subdirectory.
mkdir -p /<file_path>/paired/results/gencode/taxonomy
mkdir -p /<file_path>/paired/results/gencode/taxonomy/kraken_reports/
# Activates the "kraken" Conda environment which Kraken/Kraken2/Bracken runs within.
# The Kraken2 manual can be found at the following webpage:  https://github.com/DerrickWood/kraken2/wiki/Manual
eval "$(conda shell.bash hook)"
conda activate kraken
# Changes directories to /<file_path>/paired/results/cutadapt and establishes the naming convention for the for loop.
cd /<file_path>/paired/results/gencode/STAR/
f1=$(ls *_Unmapped.out.mate1 | sed 's/_Unmapped.out.mate1//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the _Unmapped.out.mate files within the STAR directory.
for fn in $f1
do
samplename=`basename ${fn}`
echo "Sample name is $samplename."
# Runs Kraken2 in paired-end mode with _Unmapped.out.mate input.
echo "Starting Kraken2 run for paired-end sample set $samplename using PlusPF indices."
kraken2 --threads 72 \
--db $taxon_pluspf \
--use-names \
--report-zero-counts \
--paired ${fn}_Unmapped.out.mate1 ${fn}_Unmapped.out.mate2 \
--report /<file_path>/paired/results/gencode/taxonomy/kraken_reports/${fn}_kraken.report.txt \
--output /<file_path>/paired/results/gencode/taxonomy/${fn}.kraken
done

## Step 5.2 Bracken ------------------------------------------------------------------------------------------------------------
# The Bracken manual can be found at the following webpage:  https://ccb.jhu.edu/software/bracken/index.shtml?t=manual
# Creates the Bracken reports subdirectory.
mkdir -p /<file_path>/paired/results/gencode/taxonomy/bracken/
# Changes directories to /<file_path>/paired/results/gencode/taxonomy/kraken_reports/ and establishes the naming convention for the for loop.
cd /<file_path>/paired/results/gencode/taxonomy/kraken_reports/
f1=$(ls *_kraken.report.txt | sed 's/_kraken.report.txt//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the _kraken.report.txt files within the taxonomy directory.
for fn in $f1
do
samplename=`basename ${fn}`
echo "Sample name is $samplename."
# Runs Bracken on the Kraken2 output.
echo "Starting Bracken run on the $samplename Kraken2 report."
bracken -d $taxon_pluspf \
-i ${fn}_kraken.report.txt \
-o /<file_path>/paired/results/gencode/taxonomy/bracken/${fn}.bracken \
-l S
done
# Deactivates the "kraken" Conda environment.
conda deactivate

## Step 5.3 Visualization with Krona -------------------------------------------------------------------------------------------
# The KronaTools manual can be found at the following webpage:  https://github.com/marbl/Krona/wiki/
# Changes directories to /<file_path>/paired/results/gencode/taxonomy/kraken_reports/ and establishes the naming convention for the for loop.
cd /<file_path>/paired/results/gencode/taxonomy/kraken_reports/
f1=$(ls *_kraken.report.txt | sed 's/_kraken.report.txt//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the _kraken.report.txt files within the taxonomy directory.
for fn in $f1
do
mkdir -p /<file_path>/paired/results/gencode/taxonomy/krona/${fn}/
samplename=`basename ${fn}`
echo "Sample name is $samplename."
echo "Visualizing the $samplename Kraken2 output report with Krona."
# Prepares the Kraken2 output report for input into Krona and then generates the Krona .html report.
cat /<file_path>/paired/results/gencode/taxonomy/kraken_reports/${fn}_kraken.report.txt | cut -f 2,3 > /<file_path>/paired/results/gencode/taxonomy/krona/${fn}/${fn}.kraken.krona
# Changes directories to /<file_path>/paired/results/gencode/taxonomy/krona/${fn}.
cd /<file_path>/paired/results/gencode/taxonomy/krona/${fn}/
# Generates a Krona report on the Kraken2 output report.
ktImportTaxonomy /<file_path>/paired/results/gencode/taxonomy/krona/${fn}/${fn}.kraken.krona
done

## Step 5.4 MultiQC ------------------------------------------------------------------------------------------------------------
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<file_path>/paired/results/gencode/taxonomy.
cd /<file_path>/paired/results/gencode/taxonomy/
# Runs multiqc in the current directory.
multiqc -n taxonomy_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

## Step 6 Post-Alignment QC-----------------------------------------------------------------------------------------------------  
# Performs an additional round of quality assessment on the output of the STAR alignment using Qualimap.  deepTools, Preseq, RNA-SeQC, RSeQC, Picard, and Salmon are currently not implemented.

## Step 6.1 Qualimap -----------------------------------------------------------------------------------------------------------
# Activates the "qualimap" Conda environment which Qualimap runs within.
# The Qualimap manual can be found at the following webpage:  http://qualimap.conesalab.org/doc_html/index.html
eval "$(conda shell.bash hook)"
conda activate qualimap
# Creates the directories for the output of Qualimap.
mkdir -p /<file_path>/paired/results/gencode/qualimap/
# Unsets the Qualimap display so it does not open the GUI.
unset DISPLAY
# Changes directories to /<file_path>/paired/results/gencode/STAR and establishes the naming conventions for the for loop.
cd /<file_path>/paired/results/gencode/STAR/
echo "Starting Qualimap QC on gencode Aligned.sortedByCoord.out.bam files."
f1=$(ls *_Aligned.sortedByCoord.out.bam | sed 's/_Aligned.sortedByCoord.out.bam//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the Aligned.sortedByCoord.out.bam files produced by STAR in the STAR directory.
for fn in $f1
do
samplename=`basename ${fn}`
echo "Sample name is $samplename"
mkdir -p /<file_path>/paired/results/gencode/qualimap/${fn}
# Runs Qualimap in rnaseq mode with a memory size allotment of 64 GB.
qualimap rnaseq \
-outdir /<file_path>/paired/results/gencode/qualimap/${fn} \
-a proportional \
-bam /<file_path>/paired/results/gencode/STAR/${fn}_Aligned.sortedByCoord.out.bam \
-p strand-specific-reverse \
-pe \
-s \
-gtf $gtf_gencode \
--java-mem-size=64G
done
# Deactivates the "qualimap" Conda environment.
conda deactivate

### Step 6.2 deepTools (NOT IN USE) --------------------------------------------------------------------------------------------
## Activates the "deeptools" Conda environment which deepTools runs within.
## The deepTools manual can be found at the following webpage:  https://deeptools.readthedocs.io/en/develop/content/list_of_tools.html
#eval "$(conda shell.bash hook)"
#conda activate deeptools
## Creates the directory for the output of deepTools and changes to it.
#mkdir -p /<file_path>/paired/results/gencode/deeptools/
#cd /<file_path>/paired/results/gencode/deeptools/
## Runs deepTools multiBamSummary.
#echo "Starting deepTools QC on gencode Aligned.sortedByCoord.out.bam files."
#echo "Running deepTools multiBamSummary."
#multiBamSummary bins \
#--bamfiles /<file_path>/paired/results/gencode/STAR/*_Aligned.sortedByCoord.out.bam \
#--smartLabels \
#--numberOfProcessors 72 \
#--verbose \
#--ignoreDuplicates \
#--minMappingQuality 30 \
#--outFileName readCounts.npz \
#--outRawCounts readCounts.tab
## Runs deepTools plotCorrelation using the Spearman correlation and outputs a heatmap.
#echo "Running deepTools plotCorrelation with Spearman and heatmap output."
#plotCorrelation --corData readCounts.npz \
#--corMethod spearman \
#--plotTitle "Spearman Correlation of Read Counts" \
#--whatToPlot heatmap --colorMap RdYlBu --plotNumbers \
#--plotFile heatmap_SpearmanCorr_readCounts.png \
#--outFileCorMatrix SpearmanCorr_readCounts.tab
## Runs deepTools plotPCA and outputs a PCA.
#echo "Running deepTools plotPCA."
#plotPCA	--corData readCounts.npz \
#--plotTitle "PCA of Read Counts" \
#--plotFile PCA_readCounts.png \
#--outFileNameData PCA_readCounts.tab
## Deactivates the "deeptools" Conda environment.
#conda deactivate

### Step 6.3 Preseq (NOT IN USE) -----------------------------------------------------------------------------------------------
## Activates the "preseq" Conda environment which Preseq runs within.
## The Preseq manual can be found at the following webpage:  http://smithlabresearch.org/software/preseq/
#eval "$(conda shell.bash hook)"
#conda activate preseq
## Creates the directory for the output of Preseq.
#mkdir -p /<file_path>/paired/results/gencode/preseq/
## Changes directories to /<file_path>/paired/results/gencode/STAR and establishes the naming conventions for the for loop.
#cd /<file_path>/paired/results/gencode/STAR/
#echo "Starting Preseq c_curve QC on gencode Aligned.sortedByCoord.out.bam files."
#f1=$(ls *_Aligned.sortedByCoord.out.bam | sed 's/_Aligned.sortedByCoord.out.bam//g' | sort -u)
#echo $f1
## Establishes the for loop to run through all of the Aligned.sortedByCoord.out.bam files produced by STAR in the STAR directory.
#for fn in $f1
#do
#samplename=`basename ${fn}`
#echo "Sample name is $samplename."
## Runs Preseq in c_curve mode.
#preseq c_curve -B \
#-output /<file_path>/paired/results/gencode/preseq/${fn}.txt \
#/<file_path>/paired/results/gencode/STAR/${fn}_Aligned.sortedByCoord.out.bam
#done
## Deactivates the "preseq" Conda environment.
#conda deactivate

### Step 6.4 RNA-SeQC (NOT IN USE) ---------------------------------------------------------------------------------------------
## Activates the "rna-seqc" Conda environment which RNA-SeQC runs within.
## The RNA-SeQC manual can be found at the following webpage:  https://github.com/getzlab/rnaseqc
## The GTF file must be collapsed in order to be used with this software.  Instructions on how to perform this can be found at the following webpage and a script exists to perform this action (collapse_annotation.py): https://github.com/broadinstitute/gtex-pipeline/tree/master/gene_model
#eval "$(conda shell.bash hook)"
#conda activate rna-seqc
## Creates the directory for the output of RNA-SeQC.
#mkdir -p /<file_path>/paired/results/gencode/rnaseqc/
## Changes directories to /<file_path>/paired/results/gencode/STAR and establishes the naming conventions for the for loop.
#cd /<file_path>/paired/results/gencode/STAR/
#echo "Starting RNA-SeQC QC on gencode Aligned.sortedByCoord.out.bam files."
#f1=$(ls *_Aligned.sortedByCoord.out.bam | sed 's/_Aligned.sortedByCoord.out.bam//g' | sort -u)
#echo $f1
## Establishes the for loop to run through all of the Aligned.sortedByCoord.out.bam files produced by STAR in the STAR directory.
#for fn in $f1
#do
#samplename=`basename ${fn}`
#echo "Sample name is $samplename"
## Runs RNA-SeQC.
#rnaseqc $gtf_gencode_collapsed \
#/<file_path>/paired/results/gencode/STAR/${fn}_Aligned.sortedByCoord.out.bam \
#/<file_path>/paired/results/gencode/rnaseqc/${fn} \
#-vv \
#--stranded=RF
#done
## Deactivates the "rna-seqc" Conda environment.
#conda deactivate

### Step 6.5 RSeQC (NOT IN USE) ------------------------------------------------------------------------------------------------
## Activates the "rseqc" Conda environment which RSeQC runs within.
## The RSeQC manual can be found at the following webpage:  https://rseqc.sourceforge.net/
## The GTF file must be converted to BED format in order to be used with many of the scripts within this program.  Instructions on how to perform this can be found at the following webpages and I have prepared a script to perform this action (convert_GTF_to_BED.sh):  https://www.biostars.org/p/56280/#64356 and https://www.biostars.org/p/206342/#206350 and https://bedops.readthedocs.io/en/latest/content/reference/file-management/conversion/gtf2bed.html and https://bioinformatics.stackexchange.com/questions/7094/making-a-bed-file-for-rseqc
#eval "$(conda shell.bash hook)"
#conda activate rseqc
## Creates the directory for the output of RSeQC.
#mkdir -p /<file_path>/paired/results/gencode/rseqc/inner_distance
#mkdir -p /<file_path>/paired/results/gencode/rseqc/junction_annotation
#mkdir -p /<file_path>/paired/results/gencode/rseqc/junction_saturation
## Changes directories to /<file_path>/paired/results/gencode/STAR and establishes the naming conventions for the for loop.
#cd /<file_path>/paired/results/gencode/STAR/
#echo "Starting RSeQC QC on gencode Aligned.sortedByCoord.out.bam files."
#f1=$(ls *_Aligned.sortedByCoord.out.bam | sed 's/_Aligned.sortedByCoord.out.bam//g' | sort -u)
#echo $f1
## Establishes the for loop to run through all of the Aligned.sortedByCoord.out.bam files produced by STAR in the STAR directory.
#for fn in $f1
#do
#samplename=`basename ${fn}`
#echo "Sample name is $samplename"
## Runs infer_experiment.py.
#echo "Running RSeQC infer_experiment.py."
#infer_experiment.py -i /<file_path>/paired/results/gencode/STAR/${fn}_Aligned.sortedByCoord.out.bam \
#-r $gtf_gencode_bed
## Runs read_distribution.py.
#echo "Running RSeQC read_distribution.py."
#read_distribution.py -i /<file_path>/paired/results/gencode/STAR/${fn}_Aligned.sortedByCoord.out.bam \
#-r $gtf_gencode_bed
## Runs inner_distance.py.
#echo "Running RSeQC inner_distance.py."
#inner_distance.py -i /<file_path>/paired/results/gencode/STAR/${fn}_Aligned.sortedByCoord.out.bam \
#-o /<file_path>/paired/results/gencode/rseqc/inner_distance/${fn} \
#-r $gtf_gencode_bed
## Runs junction_annotation.py.
#echo "Running RSeQC junction_annotation.py."
#junction_annotation.py -i /<file_path>/paired/results/gencode/STAR/${fn}_Aligned.sortedByCoord.out.bam \
#-o /<file_path>/paired/results/gencode/rseqc/junction_annotation/${fn} \
#-r $gtf_gencode_bed
## Runs junction_saturation.py.
#echo "Running RSeQC junction_saturation.py."
#junction_saturation.py -i /<file_path>/paired/results/gencode/STAR/${fn}_Aligned.sortedByCoord.out.bam \
#-o /<file_path>/paired/results/gencode/rseqc/junction_saturation/${fn} \
#-r $gtf_gencode_bed
#done
## Deactivates the "rseqc" Conda environment.
#conda deactivate

### Step 6.6 Picard (NOT IN USE) -----------------------------------------------------------------------------------------------
## Activates the "picard" Conda environment which Picard runs within.
## The Picard manual can be found at the following webpage:  https://broadinstitute.github.io/picard/
## The GTF file must be converted to refFlat format to work with CollectRnaSeqMetrics.  Instructions on how to do this can be found at the following webpage, and I have prepared a script to perform this action (convert_GTF_to_refFlat.sh):  https://github.com/broadinstitute/picard/issues/805 and http://genome.ucsc.edu/cgi-bin/hgTables?hgsid=1220235277_SJWvNhqI4wCU4FbY6LcxOLoaaC5l&hgta_doSchemaDb=macFas5&hgta_doSchemaTable=refFlat
## A rRNA intervals file must be generated using the GTF file.  This interval file must then be edited to have the same genome dictionary as the STAR Aligned.sortedByCoord.out.bam files.  Instructions on how to do this can be found at the following webpage, and using a script to perform this action (make_rRNA.sh):  https://www.biostars.org/p/67079/#123317 and https://github.com/broadinstitute/picard/issues/126
#eval "$(conda shell.bash hook)"
#conda activate picard
## Creates the directory for the output of Picard.
#mkdir -p /<file_path>/paired/results/gencode/picard
## Changes directories to /<file_path>/paired/results/gencode/STAR and establishes the naming conventions for the for loop.
#cd /<file_path>/paired/results/gencode/STAR/
#echo "Starting Picard QC on gencode Aligned.sortedByCoord.out.bam files."
#f1=$(ls *_Aligned.sortedByCoord.out.bam | sed 's/_Aligned.sortedByCoord.out.bam//g' | sort -u)
#echo $f1
## Establishes the for loop to run through all of the Aligned.sortedByCoord.out.bam files produced by STAR in the STAR directory.
#for fn in $f1
#do
#samplename=`basename ${fn}`
#echo "Sample name is $samplename"
## Runs CollectRnaSeqMetrics.
#echo "Running Picard Collect RnaSeqMetrics."
#picard CollectRnaSeqMetrics -I /<file_path>/paired/results/gencode/STAR/${fn}_Aligned.sortedByCoord.out.bam \
#-O /<file_path>/paired/results/gencode/picard/${fn}.RNA_metrics.txt \
#--REF_FLAT $gtf_gencode_refFlat \
#-STRAND SECOND_READ_TRANSCRIPTION_STRAND \
#--RIBOSOMAL_INTERVALS ~/Data/reference_data/gencode_GRCh38.p13.43_150bp/gencode.v43.rRNA.interval_list
#done
## Deactivates the "picard" Conda environment.
#conda deactivate

### Step 6.7 Salmon (NOT IN USE) -----------------------------------------------------------------------------------------------
## Activates the "salmon" Conda environment which Salmon runs within.
## The Salmon manual can be found at the following webpage:  https://salmon.readthedocs.io/en/latest/
## A transcriptome fasta file must be generated from the genome fasta and GTF used for alignment.  Instructions on how to do this can be found at this webpage:  http://ccb.jhu.edu/software/stringtie/gff.shtml
#eval "$(conda shell.bash hook)"
#conda activate salmon
## Creates the directories for the output of Salmon.
#mkdir -p /<file_path>/paired/results/gencode/salmon/
## Changes directories to /<file_path>/paired/results/gencode/STAR.
#cd /<file_path>/paired/results/gencode/STAR/
#echo "Starting quantification of the gencode BAM files using Salmon."
## Establishes the naming conventions for the for loop for Salmon.
#f1=$(ls *_Aligned.toTranscriptome.out.bam | sed 's/_Aligned.toTranscriptome.out.bam//g' | sort -u)
#echo $f1
## Establishes the for loop to run through all of the Aligned.sortedByCoord.out.bam files within the STAR directory.
#for fn in $f1
#do
#samplename=`basename ${fn}`
#echo "Sample name is $samplename."
#echo "Starting Salmon run for paired-end sample set $samplename using gencode reference."
#salmon quant -t $transcripts_gencode \
#-p 72 \
#-l ISR \
#-a ${fn}_Aligned.toTranscriptome.out.bam \
#-o /<file_path>/paired/results/gencode/salmon/${fn}
#done
## Deactivates the "salmon" Conda environment.
#conda deactivate

### Step 6.8 Rsem (NOT IN USE) --------------------------------------------------------------------------------------------------
## Activates the "rsem" Conda environment which Rsem runs within.
## The Rsem manual can be found at the following webpage:  https://www.encodeproject.org/documents/0c78ea4b-9392-421b-a6f3-6c858b6002aa/@@download/attachment/RSEM_Documentation.pdf
## A transcript reference directory must be generated from the genome fasta and GTF used for alignment.  Instructions on how to do this can be found by activating the "rsem" Conda environment and then typing the following command:  $ rsem-prepare-reference --help
##eval "$(conda shell.bash hook)"
##conda activate rsem
## Creates the directory for the output of Rsem.
##mkdir -p /<file_path>/paired/results/gencode/rsem
## Changes directories to /<file_path>/paired/results/gencode/STAR and establishes the naming conventions for the for loop.
##cd /<file_path>/paired/results/gencode/STAR/
##echo "Calculating expression with Rsem on gencode Aligned.toTranscriptome.out.bam files."
##f1=$(ls *_Aligned.toTranscriptome.out.bam | sed 's/_Aligned.toTranscriptome.out.bam//g' | sort -u)
##echo $f1
## Establishes the for loop to run through all of the Aligned.sortedByCoord.out.bam files produced by STAR in the STAR directory.
##for fn in $f1
##do
##samplename=`basename ${fn}`
##echo "Sample name is $samplename"
## Runs CollectRnaSeqMetrics.
##echo "Running Rsem Calculate-Expression."
##rsem-calculate-expression --paired-end --bam \
##-p 72 \
##/<file_path>/paired/results/gencode/STAR/${fn}_Aligned.toTranscriptome.out.bam \
##/home/tracy/Data/reference_data/gencode_GRCh38.p13.43_150bp/ref/gencode.v43 \
##/<file_path>/paired/results/gencode/rsem/${fn}
##done
## Deactivates the "rsem" Conda environment.
##conda deactivate

## Step 7 Feature Assignment (Subread featureCounts) ---------------------------------------------------------------------------
# Creates a feature counts matrix using the featureCounts tool from Subread from the Aligned.sortByCoord.out.bam files produced by Samtools from the STAR output.
# The Subread manual can be found at the following webpage:  https://subread.sourceforge.net/SubreadUsersGuide.pdf
# Note:  featureCounts requires that you set the strandedness parameter of the reads.  If this is set incorrectly, the rate of successfully assigned alignments will be low. 
# Creates a directory for the output of Subread and then changes directories to /<file_path>/paired/results/gencode/STAR.
mkdir -p /<file_path>/paired/results/gencode/counts/
cd /<file_path>/paired/results/gencode/STAR/
# Runs featureCounts in paired-end mode, reverse strand, with 32 threads.
echo "Running Subread featureCounts on gencode Aligned.sortedByCoord.out.bam files."
featureCounts -T 32 -s 2 -p -C -d 50 --verbose --countReadPairs -t exon -g gene_id \
-a $gtf_gencode \
-o /<file_path>/paired/results/gencode/counts/gencode_featurecounts.txt \
/<file_path>/paired/results/gencode/STAR/*_Aligned.sortedByCoord.out.bam

## Step 8 Alignment Summary (MultiQC) ------------------------------------------------------------------------------------------
# Runs MultiQC on all of the alignment and downstream files generated throughout the pipeline and collates them into a single quality assessment report.
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<file_path>/paired/results/gencode.
cd /<file_path>/paired/results/gencode/
# Runs multiqc in the current directory.
multiqc -n gencode_alignment_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

## Step 9 Pipeline Summary (MultiQC) -------------------------------------------------------------------------------------------
# Runs MultiQC on all of the files generated throughout the pipeline and collates them into a single quality assessment report.
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<file_path>/paired/results.
cd /<file_path>/paired/results/
# Runs multiqc in the current directory.
multiqc -n pipeline_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

date
